from tkinter import messagebox
import ast
import csv
def leftstr (substr = None, symbtoplit = "_"):
    """ left side of sring """
    # len of str
    lenstr = len(substr)
    # returns first occurrence of Substring 
    result = substr.find('_',1,lenstr) 
    # return left
    return left(substr,result)


def splitstrtolist(subtr, symbtoplit = "x"):
    """split str """
    return subtr.split(symbtoplit)

def converliststrtoint(liststr = None, errormsg = "can not convert to int"):
    """ conver list str to int """
    try: 
        res = list(map(int,liststr))
    except:
        messagebox.showerror ("Error",errormsg)
    return res

def left(s, amount):
    """ extract str left """
    return s[:amount]

def exstrtolistint (strp = None, symbtoplitp = "_",symbtoplitsecond = "x" ):
    """ return exstr of list int form string"""
    leftt = leftstr(substr=strp,symbtoplit=symbtoplitp)
    listtr = splitstrtolist(leftt,"x")
    return converliststrtoint (listtr,errormsg="Recheck folder name of child parent folder Folder have to template :widthxheight_infor")

def converlistinstrtolist(path = None):
    """ convert list in string to dict """
    reader = csv.reader(open(path, 'r'))
    return {k:ast.literal_eval(v)  for k,v in reader}

def Concatenatelistofstring(instr = []):
    """ Concatenate a list of strings into one string """
    return ''.join(instr)


import tkinter as tk
def crebutton (cavas,crwidth = 180 ,crheight = 480,namebutton ="FIND",*args,**kwargs):
        button1 = tk.Button(cavas, 
                      text = namebutton,
                      anchor = tk.CENTER,
                      **kwargs,
                      )
        button1_window = cavas.create_window(crwidth, crheight, 
                                              anchor=tk.CENTER, 
                                              window=button1)



import csv
class rcsv:
    """ read csv """
    def  __init__(self, pathtor = None,
                        NumberRow = 1,
                        indexarrtoget = [0,1] ) :
        self.pathtor = pathtor
        self.NumberRow = NumberRow
        self.indexarrtoget = indexarrtoget

    
    def CountNumberOfRow (self):
        """ count number of row in csv """
        with open(self.pathtor, 'r') as readFile:
            a = sum (1 for row in readFile)
        readFile.close
        return a
    
    def Redtallrowbyindxaindexarr (self):
        """ return all value in row by index """
        newRowNumber = []
        with open(self.pathtor,"r") as csvFile:
            readcsv =csv.reader(csvFile, delimiter=',')
            readcsv = list(readcsv)
            RowNumber = readcsv[self.NumberRow]
            newRowNumber = [RowNumber[ind] for ind in self.indexarrtoget]
        csvFile.close()
        return newRowNumber 

    
    def Redtallrowbyindxaindexarrall (self):
        """ return all value of all row """
        Reall = []
        recount = self.CountNumberOfRow()
        for count in range (recount):
            self.NumberRow = count
            Reall.append(self.Redtallrowbyindxaindexarr())
        return Reall

    
    def Rerowbyindxaindexarr (self):
        """ return all row follow index """
        newRowNumber = []
        with open(self.pathtor,"r") as csvFile:
                readcsv =csv.reader(csvFile, delimiter=',')
                recount = self.CountNumberOfRow()
                for count in range (recount):
                    self.NumberRow = count
                    readcsv = list(readcsv)
                    RowNumber = readcsv[self.NumberRow]
                    newRowNumber.append(RowNumber[self.indexarrtoget[0]])
        csvFile.close()
        return newRowNumber 
    
def returndictrowforcsv (path):
    """ count number of row in csv """
    with open(path, 'r') as readFile:
        listk = {lcsv[0]:lcsv[1] for lcsv in list(csv.reader(readFile, delimiter=','))}
    return listk





from pynvn.excel.list import lnumbercolumnbyrangstr
from pynvn.string.slist import returnseplistintbbystr
from pynvn.excel.rows import startrow_endrow
class hrow(object):
    """ 
    handling range for sheet\n
    rmrange: Range to handling string: \n
    ex: A1, A1:B3,A
    ws: worksheet corresponds to the rmrange \n
    option: style to  handling:\n
    ex: tspacetoospace, fs,upper_all,both, left, right,lower_all,all \n
    option_fun: For case function "REMOVESPACE or CAPFS" user select from interface \n
    ex: REMOVESPACE,CAPFS
    """
    def __init__(self,f):
        self.f = f
    def __call__(self,*args, **kwargs):
        ws = kwargs["ws"]
        rmrange = kwargs["rmrange"]
        using_value_to_end = kwargs.get("using_value_to_end",True)
        value_to_end = kwargs.get("value_to_end","End")
        valuetodelete = kwargs.get("valuetodelete",["",None])
        for rangea in rmrange:
            cols=lnumbercolumnbyrangstr(rstr=rangea)
            rows=returnseplistintbbystr(strint=rangea)
            a,b = startrow_endrow(ws=ws,
                                rows=rows,
                                cols=cols
                                )
            for col in cols:
                self.f(index_col=col,
                        ws=ws,
                        startrow=a,
                        endrow=b,
                        using_value_to_end= True,
                        valuetodelete=["",None],
                        value_to_end="VTC"
                        )
                        


import xlwings as xw
from pynvn.excel import activesheet,ws_by_namesheet
from pynvn.check.list import check_list_value
from pynvn.error import mssage_error

def cprange(pathtocopy = None,
            rangetocopy='BB5:CF100',
            pathtodes = None,
            rangetopaste="BB5", 
            namesheettocopy ='sheet_config' 
            ):
    """ 
    copy and paste range excel to another file
    ex: copy range [A1:C3] of pathtocopy to 
    C5:E20 of pathtodes
    copy to active sheet
    """
    pathtodes.range(rangetopaste).api.select
    wbtocopy = xw.Book(pathtocopy)
    sheet_copy = wbtocopy.sheets[namesheettocopy]
    sheet_copy.range(rangetocopy).api.copy
    pathtodes.api.paste
    # not cut excel 
    wbtocopy.app.api.CutCopyMode=False
    wbtocopy.close()
def cprangesamesheet(sheet = None,
                    rangetocopy='BB5:CF100',
                    rangetopaste="BB5"
                    ):

    """ copy and paste range excel to same sheet in file
    """

    sheet_des.range("{0}{1}".format(abccol,
                                    startrow)).value = fomularex
    vtformulas = sheet_des.range("{0}{1}".format(abccol,
                                                startrow)).formula
    sheet_des.range("{0}{1}:{0}{2}".format(abccol,
                                            startrow,
                                            max_row)).formula = vtformulas

def co_paste_move_range(sheet_copy = None,
            range_copy='D:D',
            sheet_des = None,
            range_paste="A1", 
            clear_rcopy_after_copy = True,
            usinglocinexcel = False
            ):
    """ 
    copy and paste range excel to sheet_des
    ex: copy range [A1:C3] of sheet_copy to 
    C5:E20 of sheet_des
    copy to active sheet
    """
    if usinglocinexcel:
        range_copy = sheet_copy.range(self.range_copy).value
        range_paste = sheet_des.range(self.range_paste).value

    if check_list_value(valuetocheck=[range_copy,range_paste]):
        sheet_des.range(range_paste).api.select
        sheet_copy.range(range_copy).api.copy
        sheet_des.api.paste
        # clear or not clear after copy
        if clear_rcopy_after_copy:
            sheet_copy.range(range_copy).clear()
    else:
        mssage_error(cont1=range_copy,cont2=range_paste)            

def cprange_2wb(pathtocopy = None,
            range_copy='D:D',
            sheet_des = None,
            range_paste="A1", 
            clear_rcopy_after_copy = True,
            sheetname_tem = ""
            ):
    """ 
    copy and paste range excel to sheet_des
    ex: copy range [A1:C3] of sheet_copy to 
    C5:E20 of sheet_des
    copy to active sheet
    """

    if check_list_value(valuetocheck=[range_copy,range_paste]):
        sheet_des.range(range_paste).api.select
        wbtocopy = xw.Book(pathtocopy,
                            update_links=False,
                            ignore_read_only_recommended=True,
                            read_only=True)
        sheet_copy = wbtocopy.sheets[sheetname_tem]
        sheet_copy.range(range_copy).api.copy
        sheet_des.api.paste
        # clear or not clear after copy
        if clear_rcopy_after_copy:
            sheet_copy.range(range_copy).clear()
    else:
        mssage_error(cont1=range_copy,cont2=range_paste)   
    wbtocopy.close()
    


class area:
    """ caculate area"""
    def __init__ (self, topleftpoint = [0,0], bottomrightpoint = [100,100], centerpoint = [0,0]):
        self.topleftpoint = topleftpoint
        self.bottomrightpoint = bottomrightpoint
        self.centerpoint = centerpoint
    
    def areafromtopbottompoint(self):
        """ caculation area from  top left and bottom right"""
        return (self.bottomrightpoint[1] - self.topleftpoint[1]) *  (self.bottomrightpoint[0] - self.topleftpoint[0])
    
    




import csv
from pynvn.stringnvn import no_accent_vietnamese
def dicttocsv (dictl = None, path = None):
    """ dict to csv """
    a_file = open(path, "w")
    writer = csv.writer(a_file,lineterminator='\n')
    for key, value in dictl.items():
        key = no_accent_vietnamese(key)
        writer.writerow([key, value])
    a_file.close()




def delrowbyindexcell (incolumndel = "C", 
                        valueofindexcoldel = None, 
                        wb = None,
                        namesheet = None,
                        startrow =1,
                        endrow = 1000,
                        valuetoendrow = "VTC"
                        ):
    """ delete row by value of cell """
    for i in range (startrow,
                        endrow):
        valuecompare =wb.sheets[namesheet].range(i,
                                                incolumndel ).value 
        k = i
        if (valuecompare == None or valuecompare == ""):
            while True:
                wb.sheets[namesheet].range('{0}:{0}'.format(k)).api.Delete(DeleteShiftDirection.xlShiftUp)
                if (wb.sheets[namesheet].range(k,incolumndel).value != None and (wb.sheets[namesheet].range(k,incolumndel).value != "")) :
                    break
        if wb.sheets[namesheet].range(k,incolumndel).value == valuetoendrow :
            break
def delrowbyrange (incolumndel = "C", 
                        valueofindexcoldel = None, 
                        ws = None,
                        startrow =1,
                        endrow = 1000,
                        value_to_end = "VTC",
                        valuetodelete = ["",None]
                        ):
    """ delete row by value of cell """
    
    for i in range (startrow,endrow):
        valuecompare =ws.range(i,incolumndel ).value 
        k = i
        if (valuecompare == None or valuecompare == ""):
            while True:
                ws.range('{0}:{0}'.format(k)).api.Delete(DeleteShiftDirection.xlShiftUp)
                if (ws.range(k,incolumndel).value != None and (ws.range(k,incolumndel).value != "")) :
                    break
        if ws.range(k,incolumndel).value == value_to_end :
            break



from pynvn.caculate.coord_point import coordp
class directnmwe:
    """ create text direction nmwe"""
    def __init__ (self,canvasb = None,
                                height = 100, 
                                width = 100,
                                dis_r = 0,
                                wr_front = 0,
                                wr_back = 0,
                                wr_left = 0,
                                wr_right = 0,
                                dis_direc = 0,
                                leftpoint = None,
                                rightpoint = None
                                ):
        self.canvasb = canvasb
        self.height = height  
        self.width = width
        self.dis_r = dis_r
        self.wr_front = wr_front
        self.wr_back = wr_back
        self.wr_left = wr_left
        self.wr_right = wr_right
        self.dis_direc = dis_direc
        self.leftpoint = leftpoint
        self.rightpoint = rightpoint

    
    def nsew(self,**kwargs):
        value_dis = self.revalue_dis()

        cp = coordp(topleftp=self.leftpoint,
                                    bottomrightp=self.rightpoint,
                                    dis_direc= self.dis_direc)

        fpc = cp.centerpoinparent()

        frontp = [fpc[0],fpc[1] - value_dis]
        backp = [fpc[0],fpc[1] + value_dis]

        leftp = [fpc[0] - value_dis ,fpc[1]]
        rightp = [fpc[0] + value_dis ,fpc[1]]

        # create text front
        try:
            self.canvasb.delete(self.frf ) # remove
        except:
            pass

        self.frf = self.canvasb.create_text(*frontp, 
                                            anchor="center",
                                            text ="Front", 
                                            angle=0,
                                            **kwargs)
        # create text back
        try:
            self.canvasb.delete(self.frb ) # remove
        except:
            pass

        self.frb = self.canvasb.create_text(*backp, 
                                            anchor="center",
                                            text ="Back", 
                                            angle=0,
                                            **kwargs)
        # create text left
        try:
            self.canvasb.delete(self.frl ) # remove
        except:
            pass

        self.frl = self.canvasb.create_text(*leftp, 
                                            anchor="center",
                                            text ="Left", 
                                            angle=90,
                                            **kwargs)
        # create text right 
        try:
            self.canvasb.delete(self.frr ) # remove
        except:
            pass

        self.frr = self.canvasb.create_text(*rightp, 
                                                anchor="center",
                                                text ="Right", 
                                                angle=-90,
                                                **kwargs)
    def revalue_dis (self):
        self.value_dis = max ([self.height/2 + self.dis_r + self.wr_front+ self.dis_direc, 
                                    self.height/2 + self.dis_r + self.wr_back + self.dis_direc,
                                    self.width/2 + self.dis_r + self.wr_left + self.dis_direc,
                                    self.width/2 + self.dis_r + self.wr_right + self.dis_direc
                                    ])
        return self.value_dis 


from pynvn.excel.list import lnumbercolumnbyrangstr
from pynvn.stringnvn.slist import returnseplistintbbystr
from pynvn.excel.del_row import delrowbyrange
from pynvn.excel.rows import startrow_endrow

class del_row(object):
    """ 
    handling range for sheet\n
    rmrange: Range to handling string: \n
    ex: A1, A1:B3,A
    ws: worksheet corresponds to the rmrange \n
    """
    def __init__(self,f):
        self.f = f
    def __call__(self,*args,**kwargs):
        self.__ws,rmrange = args
        for rangea in rmrange:
            self.__cols=lnumbercolumnbyrangstr(rstr=rangea)
            self.__rows=returnseplistintbbystr(strint=rangea)
            self.del_fun(**kwargs)

    def del_fun(self,**kwargs):
        a,b = startrow_endrow(ws=self.__ws,
                              rows=self.__rows,
                              cols=self.__cols
                              )
        for col in self.__cols:
            self.f(index_col_del=col,
                   ws=self.__ws,
                   startrow=a,
                   endrow=b,
                   **kwargs
                   )


#  write to excel 
import pandas as pd
from pynvn.path import (
                            returndatalistrowbyindex
                            )
from pynvn.data import grouper

class toexcel:
    def __init__(self,worksheet = None,
                             path = None, 
                             value = None,
                             path_conf = None,
                             lpath = None,
                             rpath = None,
                             loccelpath = 29
                             ):

        self.worksheet = worksheet
        self.value = value
        self.path = path
        self.path_conf = path_conf
        self.lpath = lpath
        self.rpath = rpath
        self.loccelpath = loccelpath

    """
    write path to location of excel 
    using path config to find address of cell
    """

    def reloccol (self):

        locpath = returndatalistrowbyindex(self.path_conf,
                                          self.loccelpath)
        # group to pair arr 
        locpath =  list(grouper(2,locpath))
        # check left and rignt to fill out to excel 
        if self.path == self.lpath:
            loc_col =  locpath[0]
        else:
            loc_col =  locpath[1]
        # convert type arr to int
        loc_col = [int(i) for i in loc_col]
        # fill out title to excel
        return loc_col
    # write move column to excel 

    def writemovecol (self,loccolmove,
                            colmove):
        loccolmove =  list(grouper(2,loccolmove))
        for cell,index in zip(loccolmove,
                                colmove):

            df = pd.read_csv(self.path,
                             delimiter=',',
                             index_col = None ,
                             usecols = [index],
                             nrows= 1)
            # convert type arr to int
            self.worksheet.cell(row = cell[0], 
                                column = cell[1]).value = df.iat[0,0]
    # write value to excell 
    def wrivaltoexc (self):
        loc_col = self.reloccol()
        self.worksheet.cell(row = loc_col[0],
                            column = loc_col[1]).value = "Path CSV"
        # fill out value path to excel 
        self.worksheet.cell(row = loc_col[0],
                            column = (int(loc_col[1]) + 1)).value = self.path



import gspread
from oauth2client.service_account import ServiceAccountCredentials
def returnclient(scope =  ["https://spreadsheets.google.com/feeds",
                            'https://www.googleapis.com/auth/spreadsheets',
                            "https://www.googleapis.com/auth/drive.file",
                            "https://www.googleapis.com/auth/drive"],
                pathjson =r"C:\Users\nhuan\Desktop\testcode\credentials.json"):
    """ return clent"""

    creds = ServiceAccountCredentials.from_json_keyfile_name(pathjson, scope)
    client = gspread.authorize(creds)
    return client



from openpyxl.styles import Alignment

from pynvn.path import ReturnDataAllRowByIndexpath,returndatalistrowbyindex

class credict():
    def __init__(self,KeyValues = None,LocConf = None,
                 Config_Setting_Path = None):
        self.Config_Setting_Path = Config_Setting_Path
        self.KeyValues = KeyValues
        self.LocConf = LocConf

    # create dict from key and value 
    def dictfromkeyandvalue(self):
        return dict (zip(self.KeyValues,
                    self.LocConf))

    # get value from conf path 
    def getvaluefromconfigpath(self):
        #list of value 
        valuelist = [[(i) for i in returndatalistrowbyindex(self.Config_Setting_Path,k)] for k in self.LocConf]
        #set new value list 
        return valuelist

    # create dicr from key and config setting path 
    def Dictfromkeyandvalueconf(self):
        valuelist = self.getvaluefromconfigpath()
        return dict (zip(self.KeyValues,
                    valuelist))

# update lists together 
def updictjoint(*List):
    Genenral_Dict = {} 
    for subDict in List:
        Genenral_Dict.update(subDict)
        gendictsorted = (sorted (Genenral_Dict.items()))
        gendictsorted = dict(gendictsorted)
    return gendictsorted

    
    
""""
         keyvalue = [ValueGeneral,
                Columnmove,
                LocationCellForMoveColumn,
                GenneralColumnNotChange,
                GeneralConcernRaffter,
                Genneral_Select,
                LocationOfRowLeft,
                LocationOfRowRight,
                ExcelCellForMoveColumnRight,
                LocationOfPurlin]

[12,16,17,19,20,21,23,24,25,26]



ValueGeneral = [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,12)]
#remove Column move 
#Columnmove = ReturnDataAllRowByIndexpath(Config_Setting_Path,32)
Columnmove = [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,16)]

LocationCellForMoveColumn = ReturnDataAllRowByIndexpath(Config_Setting_Path,17)

ArrRangMax = [* range(max([int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,11)] + Columnmove + [int(i) for i in  ReturnDataAllRowByIndexpath(Config_Setting_Path,0)]) + 1)]

ValueChangeLeft_Right  = set(ArrRangMax).difference(set(ValueGeneral + Columnmove))

IndexChangeRemoveColumn = set([int(i) for i in  ReturnDataAllRowByIndexpath(Config_Setting_Path,0)]).difference(set(Columnmove))

GenneralColumnNotChange =  [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,19)]

GeneralConcernRaffter = [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,20)]

Genneral_Select = [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,21)]

LocationOfRowLeft = [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,23)]

LocationOfRowRight = [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,24)]

ExcelCellForMoveColumnRight = ReturnDataAllRowByIndexpath(Config_Setting_Path,25)

LocationOfPurlin =  [int(i) for i in ReturnDataAllRowByIndexpath(Config_Setting_Path,26)]

TitleLocationMoveColumn = ReturnDataAllRowByIndexpath(Config_Setting_Path,27)

Index_Path_From_CSV  = ReturnDataAllRowByIndexpath(Config_Setting_Path,28)

startrow = ReturnDataAllRowByIndexpath(Config_Setting_Path,30)

"""
"""
class StringProcessing:
    def  __init__(self, path = None):
        self.path = path
        self.Return_Arr_Re = PathSteel(self.path)
        self.Arr_characters_special = self.Return_Arr_Re.ReturnDataAllRowByIndexpath(1)
        self.KeepValueNotChange = self.Return_Arr_Re.ReturnDataAllRowByIndexpath(0) 
        self.ArrReturn = self.Return_Arr_Re.ReturnDataAllRowByIndexpathAll()
        self.Check_Con_To_Case_Expect = self.Return_Arr_Re.ReturnDataAllRowByIndexpath(2)
        self.keys = self.ArrReturn[3]
    def Handling_Data_Tr (self,Arr_Index_Element,Arr):
        Arr_Index_Element = list(func(Arr_Index_Element))
        Arr_Element = [Arr[(vt[0]+1):(vt[1])] for vt in Arr_Index_Element]
        Arr_Element_El = Remove(Arr_Element)
        if "" in Arr_Element_El:
            Arr_Element_El.remove("")
        return (Arr_Element_El)
    def Handling_Data_Element(self,Arr_Elements):
        Handling_DataS_Tr_Ap = []
        for index_Arr,Arr_Element in enumerate(Arr_Elements,0):
            Arr_Index_Element = [index for index,vt in enumerate(Arr_Element,0) if vt in self.Arr_characters_special]
            Handling_DataS_Tr_eD = self.Handling_Data_Tr(Arr_Index_Element,Arr_Element)
            if str(index_Arr) in self.KeepValueNotChange:
                Handling_DataS_Tr_Ap.append(Arr_Element)
            else:
             Handling_DataS_Tr_Ap.append(Handling_DataS_Tr_eD)  
        return Handling_DataS_Tr_Ap
    def CreateDict(self):
        dictionary_Arr = []
        for i in range(4,len(self.ArrReturn),1):
            values = self.ArrReturn[i]
            Handling_Element= self.Handling_Data_Element(values)
            #Handling_Element= Handling_Data_Element(values)
            dictionary = dict(zip(self.keys, Handling_Element))
            dictionary_Arr.append(dictionary)
        return dictionary_Arr
    def Handling_DataS_Tr_For_Case_Expect(self):
        Handling_DataS_Tr_For_Case_Expected = []
        for ElementStr in self.Check_Con_To_Case_Expect:
            Arr_Index_Element = [index for index,vt in enumerate(ElementStr,0) if vt in self.Arr_characters_special]
            Value_Check_For_For_Case_Expect = self.Handling_Data_Tr(Arr_Index_Element,ElementStr)
            Handling_DataS_Tr_For_Case_Expected.append(Value_Check_For_For_Case_Expect)
        return Handling_DataS_Tr_For_Case_Expected
# remove Duplicate element from list 
def Remove(duplicate): 
    final_list = [] 
    for num in duplicate: 
        if num not in final_list: 
            final_list.append(num) 
    return final_list 
def func(alist):
    return zip(alist, alist[1:])
"""


import tkinter as tk

class dcavas:
    def __init__ (self, 
                    cavas = None, 
                    topp = None, 
                    bottomp = None):
        self.__cavas = cavas
        self.__topp = topp
        self.__bottomp = bottomp
    def drec (self, **kwargs):
        """ drawing rectangle"""
        try:

            self.__cavas.delete(self.rrectangle_kid ) # remove
        except:

            pass
        self.rrectangle_kid = self.__cavas.create_rectangle (*self.__topp,
                                                            *self.__bottomp,**kwargs)
    """
    def dimage(self):
        # put image in to layout 
        bg_icon = tk.PhotoImage(file=r"D:\5.ATADRD\CTNATAD\ctn_image\01.jpg")
        # put gif image on canvas
        # pic's upper left corner (NW) on the canvas is at x=50 y=10
        self.__cavas.create_image(100, 100, image=bg_icon, anchor=tk.NW)
    """


from tkinter import messagebox
from pynvn.excel import colnum_string
def startrow_endrow(ws = None, 
                    rows = [],
                    cols = []
                    ):
    """ 
    Retrieve start row and end row \n
    ws: worksheet input \n
    rows: can be [1], [10,200], []  \n
    if rows = [], return first and last row \n
    """
    if len(rows) == 2:
        return rows
    elif len(rows) == 1:
        return [rows[0],rows[0] + 1]
    elif len(rows) == 0:
        lr = ws.range(colnum_string(cols[0]) +\
            str(ws.cells.last_cell.row)).end('up').row
        return [1,lr + 1]
    else:
        messagebox.showerror("Error",
                             "Not find for this case rows: {0}".format(rows))



import os
import csv
import sys
from tkinter import messagebox,filedialog
import tkinter as tk
import openpyxl as xl


def check_pfile(cpath,title = "Error directory"):
    """ check directory of path """
    if cpath:
        pass
    else:
        messagebox.showerror("Error directory","Check {0} again".format(cpath))

class PathSteel:
    def  __init__(self, path_Full = None,
                        dir_path = None,
                        Is_Directory_Path_To_SubFolder = False,
                        Path_Conf = None,
                        FileName = None,
                        modulename = None,
                        pathorigrn = None,
                        pathdestination = None,
                        subfolder = None
                    ):
            self.path_Full = path_Full
            self.dir_path = dir_path
            self.Path_Conf = Path_Conf
            self.Is_Directory_Path_To_SubFolder =\
                 Is_Directory_Path_To_SubFolder
            self.FileName = FileName
            self.modulename = modulename
            self.pathorigrn = pathorigrn
            self.pathdestination = pathdestination
            self.subfolder = subfolder

    def resource_path_is_from_pyinstall_and_dev(self):
        """ Get absolute path to resource, works for dev and for PyInstaller """
        try:
            # PyInstaller creates a temp folder and stores path in _MEIPASS
            base_path = sys._MEIPASS
        except Exception:
            base_path = os.path.abspath(".")
        return os.path.join(base_path, self.FileName)

    def ExtractFileNameFromPath (self):
        """function to get faname from fullpath """
        # get file Name
        try:
            FileName = os.path.basename(self.path_Full)
        except AttributeError as error:
            print (error)
        return FileName
 
    def refpath(self,args = None):
        """Return full path conbine subfolder"""
        # check directory path is to subfolder or not ?
        if self.Is_Directory_Path_To_SubFolder == True:
            #unpack to get element in list
            #join 2 folder together 
            dir_path = os.path.join(self.dir_path,
                                            self.subfolder)
                
            # Concatenate folder and file name 
            full_path = os.path.join(dir_path,
                                    self.FileName)
        else:
            # Create full path
            full_path = os.path.join(self.dir_path,
                                    self.FileName)
            if os.path.exists(full_path) == False:
                file = open(full_path, 'w+')
        return full_path

    def getpathmodule (self):
        """ get path of module was imported """
        pathf = os.path.join((os.path.dirname(self.modulename.__file__)),
                                                self.FileName)
        return pathf
    def saveasfiletopathAndopen (self):
        # get file name from path 
        filename = ExtractFileNameFromPath(self.pathorigrn)
        wb1 = xl.load_workbook(self.pathorigrn)
        #save excel to ative folder
        try:
            wb1.save(filename)
            os.system('start "excel" {}'.format(filename))
        except:
            messagebox.showerror("Error","File name: {} is openning, close file to continue ".format(filename))
    @property
    def subfolder(self):
        return self.__subfolder
    @subfolder.setter
    def subfolder(self,newfolder):
        self.__subfolder = newfolder
    
    @property
    def FileName(self):
        return self.__FileName
    @FileName.setter
    def FileName(self,newname):
        self.__FileName = newname
    
def resource_path_is_from_pyinstall_and_dev (FileName = None,
                                             Subfolder = None ,
                                             Is_Directory_Path_To_SubFolder = False,
                                             dir_path = None
                                            ):
    """using function to return resourse path"""

    PathS = PathSteel(FileName = FileName,
                        Is_Directory_Path_To_SubFolder = Is_Directory_Path_To_SubFolder,
                        dir_path = dir_path
                    )
    return PathS.refpath(Subfolder)

def ExtractFileNameFromPath(path = None):
    """Get file name from path full
        ex:  \\folder\\nhuan.xlsx ---> nhuan.xlsx
    """
    PathS = PathSteel(path_Full = path) 
    return PathS.ExtractFileNameFromPath()

 
def IsRunningInPyinstallerBundle ():
    """running in a PyInstaller bundle or running in a normal Python process"""
    try:
        if getattr(sys, 'frozen') and hasattr(sys, '_MEIPASS'):
            return True
    except:
        return False

def PathFromFileNameAndDirpath (dir_path = None, 
                                filename = None):
    """get Path full from directory path and file name"""

    PathS = PathSteel(dir_path = dir_path,
                        FileName = filename
                    )
    full_path = PathS.refpath()
    return full_path

def getpathfromtk(outputpath = None,Warning_path_existing = True):
    """ get content entry from output for tk widget"""
    if os.path.exists(outputpath.get()):
        return outputpath.get()
    else: 
        messagebox.showinfo("Directory", "Directory not found for option") if  Warning_path_existing else False

def retabspath():
    """ return dir path folder """
    return os.path.dirname(os.path.abspath(__file__))
    
def abspath(Balance_Stock):
    return os.path.abspath(Balance_Stock)

def getdirpath(pathfull):
    """ get dir path from path full """
    return os.path.dirname(pathfull)

def getfilenamewoexten (filenameinstension):
    """ extract file name from file name """
    filename, file_extension = os.path.splitext(filenameinstension)
    return filename

def parentdirectory (path):
    """ get dir path from path full """
    return os.path.dirname (path)

def credirfol (dirNamec, subforder):
    """ create dir folder """
    try:
        dirName = os.makedirs(os.path.join(dirNamec, subforder))
    except:
        messagebox.showerror ("Error","{} already exists".format(subforder))
    return os.path.join(dirNamec, subforder)

def refullpath(dirpath, 
                filename, 
                folderchild = None, 
                createfilenameifnotexsting = True,
                createfolderifnotexsting = True):
    """ return full name from dir folder and finame name"""
    if folderchild == None:
        fpath = os.path.join(dirpath, filename)
        if os.path.exists(fpath) == False:
            open(fpath, 'w') if createfilenameifnotexsting else messagebox.showerror("name file error", 
                                                                                        "Check filename {0} in folder directory {1} not exsting  ".format(filename,
                                                                                                                                                            dirpath))
        return fpath
    else:
        if createfolderifnotexsting:
            fpathc = repathfolderchild(dirpath=dirpath,
                                            subFolder=folderchild,
                                            createfolderifnotexsting= True
                                         )
        else:
            fpathc = repathfolderchild(dirpath=dirpath,
                                            subFolder=folderchild,
                                            createfolderifnotexsting= False
                                            )
        fpath = os.path.join(fpathc, filename)
        if os.path.exists(fpath) == False:
                 open(fpath, 'w') if createfilenameifnotexsting else messagebox.showerror("name file error", 
                                                                                        "Check filename {0} in folder directory {1} not exsting  ".format(filename,
                                                                                                                                                            dirpath))
        return fpath
def repathfolderchild(dirpath, 
                        subFolder, 
                        createfolderifnotexsting = True
                        ):
    """ return path folder child from dir path and sub folder """
    path = os.path.join(dirpath,subFolder)
    if os.path.exists(path):
        return path
    else: 
        if createfolderifnotexsting:
            return credirfol(dirNamec=dirpath,subforder= subFolder)
        else:
             messagebox.showerror ("folder", "folder name {0} in path parent {1} not exists".format(subFolder,dirpath))

def resource_path_is_from_pyinstall_and_dev(relative_path):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
            base_path = sys._MEIPASS
    except Exception:
            base_path = os.path.abspath(".")
    return os.path.join(base_path, relative_path)
def listfileinfolder(path):
    """ return list file in folder"""
    try: 
        os.chdir(path)
        # list file  excel 
        return os.listdir(path)
    except:
        messagebox.showerror ("Error","check for folder path {}".format(path))

def mfileopen(outputtk, 
                file_open = True):
        """ open file parent"""
        outputtk.delete(0, 'end')
        if file_open:
            # ask directory file
            files = filedialog.askopenfilename(title = "Directory of parent file",
                                                initialdir=outputtk.get())

            outputtk.insert(tk.END,files)

        else:
            # ask directory folder
            files = filedialog.askdirectory(title = "Directory of folder",
                                            initialdir=outputtk.get())

            outputtk.insert(tk.END,files)


def p_pyinstall_and_dev(relative_path):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
            base_path = sys._MEIPASS
    except Exception:
            base_path = os.path.abspath(".")
    return os.path.join(base_path, relative_path)
def removeexfilename(filename):
    """
    remove extension from file name
    EX:mmm.csv -----> mmm
    """
    return os.path.splitext(filename)[0]


def dirfolder (dirNamec, subforder,alertexists = True):
    """ create dir folder """
    try:
        dirName = os.makedirs(os.path.join(dirNamec, subforder))
    except:
        if alertexists:
            messagebox.showerror ("Error","{} already exists".format(subforder))
        else:
            pass
    return os.path.join(dirNamec, 
                        subforder)

def file_open(outputtk):
    """ ask directory file """
    outputtk.delete(0, 'end')    
    files = filedialog.askopenfilename(title = "Directory of parent file",
                                        initialdir=outputtk.get())

    outputtk.insert(tk.END,files)

def ask_directory(outputtk):
    """ ask directory """
    outputtk.delete(0, 'end')    
    folder = filedialog.askdirectory(title = "Directory of folder",
                                            initialdir=outputtk.get())
                                            
    outputtk.insert(tk.END,folder)


def ask_open(outputtk, 
                open_file = True):
        """ open directory file or folder"""
        outputtk.delete(0, 'end')
        file_open(outputtk) if open_file else ask_directory(outputtk)


import xlwings as xw
from pynvn.list import listpairfrom2list
from pynvn.csv.tocsv import pairlistinlisttocsv
copyxw = xw.Book(r'C:\Users\HP\Desktop\exazb\conf_ex.xlsx')
copyxws = copyxw.sheets
wsc = copyxws["hrdata_modified"]
def removevaluenontinlistpair(lista,deleteifvalue = [None]):
    """
    pairlistremoved = []
    for pairarr in lista:
        if pairarr[0] not in deleteifvalue:
            pairlistremoved.append(pairarr)
    return pairlistremoved
    """
    return [pairarr for pairarr in lista if pairarr[0] not in deleteifvalue]

def pairlistfromexcel (startrow= 1, 
                        floc = "A", 
                        sloc = "B",
                        sheet = None    
                        ):
    """ create pair list from floc and sloc of excel """
    # max row at floc 
    m_rowatfloc = sheet.range(floc + str(sheet.cells.last_cell.row)).end('up').row
    # create list from range at floc 
    listfloc = sheet.range("{0}{1}:{0}{2}".format(floc,startrow,m_rowatfloc)).value
    # create list from range at sloc 
    listsloc = sheet.range("{0}{1}:{0}{2}".format(sloc,startrow,m_rowatfloc)).value

    return listpairfrom2list(list_a=listfloc,
                            list_b=listsloc)


lissta = pairlistfromexcel(sheet=wsc)

paira = removevaluenontinlistpair(lista=lissta)

pairlistinlisttocsv(listvalue=lissta,pathcsv=r'C:\Users\HP\Desktop\exazb\conf_ex.csv')



from tkinter import messagebox
class loginatad:
    "input user and password to sw"
    def __init__ (self,uname= None,pass_ = None):
        self.uname = uname
        self.pass_ = pass_
    # check user and password when login 
    def login (self):
        if self.uname.get() =="" or self.pass_.get() =="":
            messagebox.showerror ("Error",
                                "All fields are required !!")
            return "success"
                
        elif self.uname.get() =="nhuan" and self.pass_.get() =="1234":
            
            return "success"

        else:
            messagebox.showerror ("Error",
                                "invalid username or password")
            return "success"
    


from pynvn.path.ppath import (
                            refullpath,
                            removeexfilename,
                            dirfolder,p_pyinstall_and_dev
                            )
from pynvn.folder.list import file_in_folder

def rdict_path(dirpath = None, 
                folderchild = "config",
                namefiles = ["logo.png","hrdata_modified.csv","conf_ex.xlsx","config_hm.xlsx","key.key","fn.csv","seri.key","ser.csv"],
                keepextension = False
                ):
    """ return dict of path """
    dict_path = {}
    for namefile in namefiles:
        if keepextension:
            keyd = folderchild + "_" + namefile
        else:
            keyd = folderchild + "_" + removeexfilename(namefile)

        keyd = folderchild + "_" + removeexfilename(namefile)
        dict_path[keyd] =  p_pyinstall_and_dev(refullpath(dirpath=dirpath,
                                            folderchild=folderchild, 
                                            filename=namefile
                                            ))
    return dict_path

def rdict_fleinfolder(dirpath = None, 
                    folderchild = ["config","img"],
                    keepextension = False
                    ):
    """ 
    return dict of paths 
    get file in folder

    """
    rda = {}    
    for dch in  folderchild:
        dfer = dirfolder(dirNamec=dirpath,
                        subforder = dch,
                        alertexists=False
                        )
        lfname = file_in_folder(dfer)
        rd = rdict_path(dirpath=dirpath,
                    folderchild=dch,
                    namefiles=lfname,keepextension=keepextension)
        rda.update(rd)

    return rda


def rdict_file_in_folder(dirpath = None, 
                    folderchild = ["config","img"],
                    keepextension = False
                    ):
    """ 
    return dict of paths 
    get file in folder

    """
    rda = {}    
    for dch in  folderchild:
        dfer = dirfolder(dirNamec=dirpath,
                        subforder = dch,
                        alertexists=False
                        )
        lfname = file_in_folder(dfer)
        rd = rdict_path(dirpath=dirpath,
                    folderchild=dch,
                    namefiles=lfname,keepextension=keepextension)
        rda.update(rd)

    return rda


import ast
import csv
import re
def strincsvtodict(path = None):
    """ convert list in string to dict """
    reader = csv.reader(open(path, 'r'))
    return {k:ast.literal_eval(v)  for k,v in reader}

def returndictrowforcsv (path):
    """ count number of row in csv """
    with open(path, 'r') as readFile:
        listk = {lcsv[0]:lcsv[1] for lcsv in list(csv.reader(readFile, delimiter=','))}
    return listk


def dict_str_fromlist(path = None):
    """ 
    convert list in string to dict 
    ex: csv content: "r,A1,A2,A3" ---> dict: {r: ["A1","A2","A3"}
    
    """
    reader = csv.reader(open(path, 'r'))
    return {k:re.split("[,]", v)  for k,v in reader}


def dict_from_csv2col (path):
    """ retriver dict from two column in csv """
    with open(path, 'r') as readFile:
        listk = {lcsv[0]:lcsv[1] for lcsv in list(csv.reader(readFile, delimiter=','))}
    return listk

def evallist(n):
    try:
        return eval(n)
    except:
        return n

def dictfromcsv2col_evallist(path):
    """ retrieve dict from two column in csv """
    with open(path, 'r') as readFile:
        listk = {lcsv[0]:evallist(lcsv[1]) for lcsv in list(csv.reader(readFile, delimiter=','))}
    return listk

def dictFcsv_2col_eval_list(path):
    """ retrieve dict from two column in csv """
    with open(path, 'r') as readFile:
        listk = {lcsv[0]:evallist(lcsv[1]) for lcsv in list(csv.reader(readFile, delimiter=','))}
    return listk

def dict_str_from_lcsv(path = None):
    """ 
    convert list in string to dict 
    ex: csv content: "r,A1,A2,A3" ---> dict: {r: ["A1","A2","A3"}
    
    """
    reader = csv.reader(open(path, 'r'))
    return {k:re.split("[,]", v)  for k,v in reader}


import os
from pynvn.path.ppath import dirfolder
def listfileofpfolder(folderchild, fmnamefile = ["jpg","gif"]):
    """ return list file of p folder"""
    os.chdir(folderchild)
    lfiles = os.listdir(folderchild)
    lfileinforders = [lfile for lfile in lfiles if any(n in lfile for n in fmnamefile)]
    return lfileinforders
def file_in_folder(path):
    """ return list file in folder"""
    try: 
        os.chdir(path)
        # list file  excel 
        return os.listdir(path)
    except:
        messagebox.showerror ("Error","check for folder path {}".format(path))
def file_in_folders(paths = []):
    """ return list file in folders """
    flist = []
    for path in paths:
        elist = file_in_folder(path)
        flist = flist + elist
    return flist

def ldirfolders(dirNamec = None, 
                lsubforders = [],
                alertexists = True
                ):
    """ return list dir folders """
    return [dirfolder(dirNamec=dirNamec,subforder=subf,alertexists=alertexists) for subf in lsubforders]


def forget(widget): 
    """ invisible gird and pack """
    # This will remove the widget from toplevel 
    #basically widget do not get deleted 
    # it just becomes invisible and loses its position 
    # and can be retrieve 
    widget.grid_forget()        
def return_gird_pack(widget,
                    insidepackorgird = "gird",
                    **kw):
    """ visible gird and pack """
    widget.grid(**kw) if insidepackorgird == "gird" else widget.pack(**kw)




import tkinter as tk
from pynvn.autoscrollbar.autoscrbar import AutoScrollbar

class ChecklistBox:
    """return check list box"""
    def __init__(self, parent, 
                choices = ["Nhuan", "Hong", "Trum","Nhuan", "Hong", "Trum"], 
                i = 1,
                onvalue = True,
                offvalue = "" ,
                width = 60,
                listsheetname = None,
                midstr = "_ Sheet name: ",
                texttitle = "File Name _ sheet Name",
                **kwargs):
        self.choices = choices
        self.onvalue = onvalue
        self.offvalue = offvalue
        self.parent = parent
        self.vars = []
        self.i = i
        self.width = width
        self.listsheetname = listsheetname
        self.midstr = midstr
        self.texttitle = texttitle
        self.__rechecklistbox()
    def __rechecklistbox (self):
        """ return check list box from arr """
        for idx,choice in enumerate(self.choices):
            var = tk.StringVar()

            self.vars.append(var)
            listname = "" if self.listsheetname == None else self.listsheetname[idx]
            if self.texttitle != "":
                lb = tk.Label (self.parent,text = self.texttitle,bg = "white")
                lb.grid(row=0,
                        column=0, 
                        sticky =  tk.W)
            cb = tk.Checkbutton(self.parent, 
                                var=var, 
                                text=choice + self.midstr + listname,
                                onvalue=choice,
                                offvalue=self.offvalue,
                                anchor=tk.W, 
                                bg = "azure",
                                relief="flat", 
                                highlightthickness=0
                                )
            cb.grid(row=self.i,
                        column=0,
                        sticky = tk.NSEW,
                        ) 
            self.i = self.i + 1

    def getCheckedItems(self):
        """ get checked items """
        return [var.get() for var in self.vars if var.get() !=""]


def check_list_value (valuetocheck = ["None","nhuan"], 
                        checkvalue = [None,""],
                        not_in_checkvalue = True
                        ):
    """ 
    check value in list using all 
    ex1 = valuetocheck = [None,"nhuan"] heckvalue = [None,""] return False,
    ex2 = valuetocheck = ["None","nhuan"] heckvalue = [None,""] return True

    """
    if not_in_checkvalue:
        boresult = all(n not in checkvalue for n in valuetocheck)
    else:
        boresult = any(n in checkvalue for n in valuetocheck)
    return boresult


from pynvn.excel import col2num   
import openpyxl
def returnloccellbyvalue(path = None ,min_row = 1,max_row=100,min_col=3,max_col=3,valuetofind = 1200, namesheet = None):
    """ return location of cell (row index and column index) ---->[cell.row,cell.column]"""
    if type(min_col) == str:
        min_col = col2num(min_col)
    if type(max_col) == str:
        max_col = col2num(max_col)
    wb = openpyxl.load_workbook(path, 
                                read_only=True)
    ws = wb[namesheet]
    for row in ws.iter_rows(min_row = min_row,
                            max_row=max_row,
                            min_col=min_col,
                            max_col=max_col):
        for cell in row:
            if str(cell.value) == str(valuetofind):
                return [cell.row,cell.column] #change column number for any cell value you wants


from pynvn import dict_str_from_lcsv
from pynvn.excel import (sheet_by_namesheet,
                        activesheet
                        )
from pynvn import filter_lstr
from tkinter import messagebox
from appnvn.RetrFile.hstr import hstr_ex
from pynvn.string import twospace_to_onespace
@hstr_ex
def removespace (
                instr = None, 
                option = "both"
                ):
    """
    remove space from string \n
    instr: input your string \n
    option: stype handling string \n
    """
    rspace_fun = {
                "---> name STYLE from config <---": lambda:  "-------------> Input your function <---------------",
                "---> name STYLE from config <--- ": lambda:  "-------------> Input your function <---------------",
                }
    return rspace_fun[option]()

class rapp:
    """ 
    Generic class for this template with variables\n
    retr_path: directory of execute excel file\n
    retr_sheetname: name of sheet name of retr_path excel\n
    fuction: choose the feature  want to use \n
    pathconf: Directory of path conf (.csv), this path have conf parameter 
    """
    def __init__(self,
                retr_path = None,
                retr_sheetname =None, 
                fuction = "",
                pathconf = None,
                ):
        self.__retr_sheetname = retr_sheetname
        self.__fuction = fuction.lower()
        # Create a dict have parameter from csv 
        self.__dictconf = dict_str_from_lcsv(path=pathconf)
        # Option from user input (Active Sheet or not)
        self.__ws_retr =  activesheet() if retr_sheetname == "Active Sheet"\
                                        else sheet_by_namesheet(path=retr_path,
                                                                namesheet=retr_sheetname
                                                                )
        # Check file excel execute is conf_ex.xlsx or not 
        if self.__ws_retr.name == "hrdata_modified":
            messagebox.showerror("Error Name Excel",
                                "Can not using file excel: conf_ex.xlsx to execute. \
                                Check again {}".format("hrdata_modified")
                                )

    def ft_tool(self):
        """
        execute func of sortware \n
        ex: removespace, capfs

        """
        # filter list string only key from csv
        lfuns = filter_lstr(liststr=list(self.__dictconf.keys()),
                                        reverse_criteria=True,
                                        criteria=["sub_"],
                                        upper = False
                                        )
        # create dict have fun execute 
        mydictfun = {
                    "removespace":(lambda: self.__removespace()),
                    "capfs":(lambda: self.__capfs())
                    }
        # Option from user input function (config or not)
        if self.__fuction == "Config":
            for lfun in lfuns:
                mydictfun[lfun]()
        else:
            mydictfun[self.__fuction]()
    
    def __removespace(self):

        """ 
        For case function "REMOVESPACE" user select from interface 
        """
        cyesornot = self.__dictconf["removespace"]
        rmrange = self.__dictconf["sub_removespace_range"]
        rmstype = self.__dictconf["sub_removespace_style"]
        removespace(ws =self.__ws_retr,
                    rmrange=rmrange, 
                    option=rmstype
                    ) if cyesornot[0] =="yes" else False  
    def __capfs(self):
        """ 
        For case function "CAPFS" user select from interface 
        """
        cyesornot = self.__dictconf["capfs"]
        rmrange = self.__dictconf["sub_capfs_range"]
        rmstype = self.__dictconf["sub_capfs_style"]
        capitalize(rmrange=rmrange,
                        option= rmstype,
                        ws=self.__ws_retr,
                        ) if cyesornot[0] =="yes" else False



from tkinter import messagebox
import xlwings as xw 
from pynvn.csv.rcsv import returndictrowforcsv
from pynvn.stringnvn.slist import list_from_listinstr
from appnvn.exazp.excel.hchildsheet import hchildsheet

class cexcel:
    """copy excel to excel"""
    def __init__(self,
                pathconf = None,
                pathdes= None, 
                pathtocopy= None,
                ):
        # call dict 
        self.dictconf = returndictrowforcsv(path=pathconf)
        # return list sheet excel to handling 
        self.__pathdes = pathdes
        self.__pathtocopy = pathtocopy
        self.__app = xw.App(visible=True,
                            add_book=False
                            )
        self.__desxw = self.__app.books.open(fullname=self.__pathdes,
                                            update_links=False)

        self.__copyxw = self.__app.books.open(fullname=self.__pathtocopy,
                                            update_links=False,                       
                                            read_only=False,
                                            ignore_read_only_recommended=False
                                            )
        self.wsnames = self.__copyxw.sheets
        for namesheet in self.wsnames:
            if "AZB" in namesheet.name:
                self.ws1 = namesheet
                break
        # list sheet name 
        listsheetex = list_from_listinstr(self.dictconf["listsheetnamechild"].replace(":", ","))
        # name sheet 
        self.__namesheet = self.ws1.name
        # set active sheet name 
        self.__desxw.sheets[self.__namesheet].activate()
        print("sheet name" ,self.__namesheet)
        # check name sheet name 
        if self.__namesheet not in listsheetex:
            messagebox.showerror("error", "name sheet {0} of workbook{1} not valid, its \
                                    name is AZB-NN".format(self.__namesheet,pathtocopy)
                                )
    def copysheettoexcelexist(self):
        """ copy sheet name  to excel existing """
        sheet_des = self.__desxw.sheets[self.__namesheet]
        sheet_copy = self.__copyxw.sheets[self.__namesheet]

        if self.__namesheet == "AZB-10":
            recor_l_lint = int(self.dictconf["zab10_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab10_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab10_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb10_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab10_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab10_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab10_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup
                                )
    
        if self.__namesheet == "AZB-20":
            recor_l_lint = int(self.dictconf["zab20_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab20_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab20_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb20_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab20_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab20_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab20_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup)
        if self.__namesheet == "AZB-30":
            recor_l_lint = int(self.dictconf["zab30_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab30_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab30_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb30_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab30_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab30_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab30_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup)

        if self.__namesheet == "AZB-40":
            recor_l_lint = int(self.dictconf["zab40_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab40_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab40_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb40_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab40_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab40_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab40_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup
                                )
        if self.__namesheet == "AZB-50":
            recor_l_lint = int(self.dictconf["zab50_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab50_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab50_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb50_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab50_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab50_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab50_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup
                                )

        if self.__namesheet == "AZB-60":
            recor_l_lint = int(self.dictconf["zab60_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab60_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab60_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb60_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab60_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab60_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab60_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup
                                )

        if self.__namesheet == "AZB-70":
            recor_l_lint = int(self.dictconf["zab70_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab70_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab70_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb70_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab70_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab70_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab70_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup
                                )
        if self.__namesheet == "AZB-80":
            recor_l_lint = int(self.dictconf["zab80_recor_l1"])
            valueim = list_from_listinstr(self.dictconf["zab80_valueim"].replace(":", ","))
            valuehavechild = list_from_listinstr(self.dictconf["zab80_valuehavechild"].replace(":", ","))
            msstr =self.dictconf["azb80_ms"]
            forbydup = list_from_listinstr(self.dictconf["zab80_forbydup"].replace(":", ","))
            locuseformulas = list_from_listinstr(self.dictconf["zab80_locuseformulas"].replace(":", ","))
            col_dup = list_from_listinstr(self.dictconf["zab80_dup"].replace(":", ","))
            hchild = hchildsheet(startrow=recor_l_lint,
                                col_key_msa=msstr,
                                lcolumnformulas = locuseformulas,
                                valueim=valueim,
                                sheet_des =sheet_des,
                                sheet_copy=sheet_copy,
                                col_dup=col_dup,
                                formulasfor_col_dup = forbydup
                                )
        self.__copyxw.close()
        self.__desxw.save()
        self.__desxw.close()
        self.__app.quit()


from pynvn.caculate.ratio import ratio
from PIL import Image, ImageTk
import warnings
import math
class zmcv:
    """ scale, move in cavas """
    def __init__(self, cavas,frameb,
                    value_dis = 1000, 
                    distancezx = 0,
                    distancezy = 0 ,
                    centerp =[0,0], 
                    usingcoord = False,
                    imageid = None,
                    arealayoutwh = [4000,4000],
                    imagepath = None,
                    isimage = False
                    ):
        self.arealayoutwh = arealayoutwh
        self.usingcoord = usingcoord
        self.cavas = cavas
        self.frameb = frameb
        self.value_dis = value_dis
        self.distancezx = distancezx
        self.distancezy = distancezy
        self.centerp = centerp
        self.imageid = imageid
        self.delta = 0.9
        self.imagepath = imagepath
        self.imscale = 1.0  # scale for the canvas image zoom, public for outer classes
        self.__delta = 1.3  # zoom magnitude
        self.__filter = Image.ANTIALIAS  # could be: NEAREST, BILINEAR, BICUBIC and ANTIALIAS
        self.__previous_state = 0  # previous state of the keyboard
        self.path = self.imagepath  # path to the image, should be public for outer classes
        # Decide if this image huge or not
        self.minradio = ratio(real_w=self.frameb[2],
                                    real_h=self.frameb[3],
                                    w = self.value_dis * 2,
                                    h = self.value_dis * 2)\
                                    .reratiomin()
        if self.usingcoord:
            self.cavas.bind("<MouseWheel>",self.zoomer)
            # This is what enables using the mouse:
            self.cavas.bind("<ButtonPress-1>", self.move_start)
            self.cavas.bind("<B1-Motion>", self.move_move)
            self.sccv()

        elif isimage:
            # Bind events to the Canvas
            self.cavas.bind('<Configure>', lambda event: self.__show_image())  # canvas is resized
            self.cavas.bind('<ButtonPress-1>', self.__move_from)  # remember canvas position
            self.cavas.bind('<B1-Motion>',     self.__move_to)  # move canvas to the new position
            self.cavas.bind('<MouseWheel>', self.__wheel)  # zoom for Windows and MacOS, but not Linux
            self.cavas.bind('<Button-5>',   self.__wheel)  # zoom for Linux, wheel scroll down
            self.cavas.bind('<Button-4>',   self.__wheel)  # zoom for Linux, wheel scroll up
            # Handle keystrokes in idle mode, because program slows down on a weak computers,
            # when too many key stroke events in the same time
            self.cavas.bind('<Key>', lambda event: self.cavas.after_idle(self.__keystroke, 
                                                                                    event)
                                                                                        )

            self.__image = Image.open(self.path)  # open image, but down't load it
            self.imwidth, self.imheight = self.__image.size  # public for outer classes

            self.__min_side = min(self.imwidth, 
                                    self.imheight)  # get the smaller image side
            # Create image pyramid
            self.__pyramid = [Image.open(self.path)]
            # Set ratio coefficient for image pyramid
            self.__ratio =  1
            self.__curr_img = 0  # current image from the pyramid
            #self.__scale =  self.imscale * self.__ratio  # image pyramide scale
            self.__reduction = 45  # reduction degree of image pyramid
            w, h = self.__pyramid[-1].size

            wr, hr = self.arealayoutwh
            
            self.__returnreduction(max(wr, hr))

            self.ratio = min(wr/w,hr/h) 
            retioimage = w/h 
            if wr <= hr:
                self.ratio = min(wr/w,hr/h)
                hr = wr/ retioimage    
            else:
                self.ratio = min(wr/w,hr/h)
                wr =  hr * retioimage
            hr = wr/ retioimage    
            while w > 512 and h > 512:  # top pyramid image is around 512 pixels in size
                w /= self.__reduction  # divide on reduction degree
                h /= self.__reduction  # divide on reduction degree
                self.__pyramid.append(self.__pyramid[-1].resize((int(w),
                                                                 int(h)), 
                                                                 self.__filter))
            
            # top left and button right of rec coord
            coordrec = [self.centerp[0] - wr/2,
                        self.centerp[1] - hr/2,
                        self.centerp[0] + wr/2,
                        self.centerp[1] + hr/2]
            # Put image into container rectangle and use it to set proper coordinates to the image
            self.container = self.cavas.create_rectangle((coordrec), 
                                                        width=0,
                                                        fill="blue"
                                                        )
            self.sccv()
            self.__scale =  self.minradio /1.1
            self.__show_image()  # show image on the canvas
            self.cavas.focus_set()  # set focus on the canvas

        else:
            self.cavas.bind("<MouseWheel>",self.zoomer)
            # This is what enables using the mouse:
            self.cavas.bind("<ButtonPress-1>", self.move_start)
            self.cavas.bind("<B1-Motion>", self.move_move)
            self.sccv()

    def zoomer(self,event):
        """windows zoom """
        if self.usingcoord == False:
            if (event.delta > 0):
                    self.cavas.scale("all",
                                        self.frameb[2]/2 - self.distancezx, 
                                        self.frameb[3]/2 - self.distancezy, 
                                        1.1, 
                                        1.1)
            elif (event.delta < 0):
                    self.cavas.scale("all", 
                                        self.frameb[2]/2 - self.distancezx, 
                                        self.frameb[3]/2 - self.distancezy ,
                                        0.9, 
                                        0.9)
                    self.cavas.configure(scrollregion = self.cavas.bbox("all"))
        else:
            if (event.delta > 0):
                self.cavas.scale("all",
                                *self.centerp , 
                                1.1, 
                                1.1)
            elif (event.delta < 0):
                self.cavas.scale("all",
                                *self.centerp , 
                                0.9, 
                                0.9)
                self.cavas.configure(scrollregion = self.cavas.bbox("all"))
    
    #move
    def move_start(self, event):
                self.cavas.scan_mark(event.x, 
                                    event.y)
    def move_move(self, event):
                self.cavas.scan_dragto(event.x, 
                                        event.y, 
                                        gain=1) 
    
    def sccv (self):
        """ scale in cavas """
        if self.usingcoord:
            self.cavas.scale("all",self.centerp[0], 
                                    self.centerp[1], 
                                    self.minradio/1.1, 
                                self.minradio/1.1)
        else:
            self.cavas.scale("all",self.frameb[2]/2 - self.distancezx, 
                                self.frameb[3]/2- self.distancezy, 
                                self.minradio/1.1, 
                                self.minradio/1.1)

    def redraw_figures(self):
        """ Dummy function to redraw figures in the children classes """
        pass

    def grid(self, **kw):
        """ Put CanvasImage widget on the parent widget """
        self.__imframe.grid(**kw)  # place CanvasImage widget on the grid
        self.__imframe.grid(sticky='nswe')  # make frame container sticky
        self.__imframe.rowconfigure(0, 
                                    weight=1)  # make canvas expandable
        self.__imframe.columnconfigure(0, 
                                        weight=1)

    def pack(self, **kw):
        """ Exception: cannot use pack with this widget """
        raise Exception('Cannot use pack with the widget ' + self.__class__.__name__)

    def place(self, **kw):
        """ Exception: cannot use place with this widget """
        raise Exception('Cannot use place with the widget ' + self.__class__.__name__)

    # noinspection PyUnusedLocal
    def __scroll_x(self, *args, **kwargs):
        """ Scroll canvas horizontally and redraw the image """
        self.cavas.xview(*args)  # scroll horizontally
        self.__show_image()  # redraw the image

    # noinspection PyUnusedLocal
    def __scroll_y(self, *args, **kwargs):
        """ Scroll canvas vertically and redraw the image """
        self.cavas.yview(*args)  # scroll vertically
        self.__show_image()  # redraw the image

    def __show_image(self):
        """ Show image on the Canvas. Implements correct image zoom almost like in Google Maps """
        box_image = self.cavas.coords(self.container)  # get image area
        box_canvas = (self.cavas.canvasx(0),  # get visible area of the canvas
                      self.cavas.canvasy(0),
                      self.cavas.canvasx(self.cavas.winfo_width()),
                      self.cavas.canvasy(self.cavas.winfo_height()))
        box_img_int = tuple(map(int, box_image))  # convert to integer or it will not work properly
        # Get scroll region box
        box_scroll = [min(box_img_int[0], 
                        box_canvas[0]),
                        min(box_img_int[1], 
                        box_canvas[1]),
                        max(box_img_int[2], 
                        box_canvas[2]), 
                        max(box_img_int[3], 
                        box_canvas[3])]
        # Horizontal part of the image is in the visible area
        if  box_scroll[0] == box_canvas[0] and box_scroll[2] == box_canvas[2]:
            box_scroll[0]  = box_img_int[0]
            box_scroll[2]  = box_img_int[2]
        # Vertical part of the image is in the visible area
        if  box_scroll[1] == box_canvas[1] and box_scroll[3] == box_canvas[3]:
            box_scroll[1]  = box_img_int[1]
            box_scroll[3]  = box_img_int[3]
        # Convert scroll region to tuple and to integer
        self.cavas.configure(scrollregion=tuple(map(int, box_scroll)))  # set scroll region
        x1 = max(box_canvas[0] - box_image[0], 0)  # get coordinates (x1,y1,x2,y2) of the image tile
        y1 = max(box_canvas[1] - box_image[1], 0)
        x2 = min(box_canvas[2], box_image[2]) - box_image[0]
        y2 = min(box_canvas[3], box_image[3]) - box_image[1]
        if int(x2 - x1) > 0 and int(y2 - y1) > 0:  # show image if it in the visible area
            image = self.__pyramid[max(0, self.__curr_img)].crop(  # crop current img from pyramid
                                                                (int(x1 / (self.__scale*self.ratio)),
                                                                int(y1 / (self.__scale*self.ratio)),
                                                                int(x2 / (self.__scale*self.ratio)),
                                                                int(y2 / (self.__scale*self.ratio))))
            
            # resize image
            imagetk = ImageTk.PhotoImage(image.resize((int(x2 - x1), 
                                                        int(y2 - y1)), 
                                                        self.__filter))
            
            imageid = self.cavas.create_image(max(box_canvas[0],
                                                box_img_int[0]),
                                                max(box_canvas[1], 
                                                box_img_int[1]),
                                                anchor='nw', 
                                                image=imagetk)
            self.cavas.imagetk = imagetk  # keep an extra reference to prevent garbage-collection

    def __move_from(self, event):
            """ Remember previous coordinates for scrolling with the mouse """
            self.cavas.scan_mark(event.x, event.y)

    def __move_to(self, event):
        """ Drag (move) canvas to the new position """
        self.cavas.scan_dragto(event.x, event.y, gain=1)
        self.__show_image()  # zoom tile and show it on the canvas

    def outside(self, x, y):
        """ Checks if the point (x,y) is outside the image area """
        bbox = self.cavas.coords(self.container)  # get image area
        if bbox[0] < x < bbox[2] and bbox[1] < y < bbox[3]:
            return False  # point (x,y) is inside the image area
        else:
            return True  # point (x,y) is outside the image area

    def __wheel(self, event):
        """ Zoom with mouse wheel """
        x = self.cavas.canvasx(event.x)  # get coordinates of the event on the canvas
        y = self.cavas.canvasy(event.y)
        if self.outside(x, y): return  # zoom only inside image area
        scale  = 1.0
        # Respond to Linux (event.num) or Windows (event.delta) wheel event
        if event.num == 5 or event.delta == -120:  # scroll down, smaller
            if round(self.__min_side * self.imscale) < 30: return  # image is less than 30 pixels
            self.imscale /= self.__delta
            scale        /= self.__delta
        if event.num == 4 or event.delta == 120:  # scroll up, bigger
            i = min(self.cavas.winfo_width(), self.cavas.winfo_height()) >> 1
            if i < self.imscale: return  # 1 pixel is bigger than the visible area
            self.imscale *= self.__delta
            scale        *= self.__delta
        # Take appropriate image from the pyramid
        k = self.imscale * self.__ratio * self.minradio  # temporary coefficient
        self.__curr_img = min((-1) * int(math.log(k, self.__reduction)), 
                            len(self.__pyramid) - 1)
        self.__scale = k * math.pow(self.__reduction, 
                                max(0, self.__curr_img))/1.1
        #
        self.cavas.scale('all', 
                            x, 
                            y, 
                            scale, 
                            scale)  # rescale all objects
        # Redraw some figures before showing image on the screen
        self.redraw_figures()  # method for child classes
        self.__show_image()

    def __keystroke(self, event):
        """ Scrolling with the keyboard.
            Independent from the language of the keyboard, CapsLock, <Ctrl>+<key>, etc. 
        """
        if event.state - self.__previous_state == 4:  # means that the Control key is pressed
            pass  # do nothing if Control key is pressed
        else:
            self.__previous_state = event.state  # remember the last keystroke state
            # Up, Down, Left, Right keystrokes
            if event.keycode in [68, 39, 102]:  # scroll right, keys 'd' or 'Right'
                self.__scroll_x('scroll',  
                                1, 
                                'unit', 
                                event=event
                                )

            elif event.keycode in [65, 37, 100]:  # scroll left, keys 'a' or 'Left'
                self.__scroll_x('scroll',
                                     -1, 
                                     'unit', 
                                     event=event
                                     )
            elif event.keycode in [87, 38, 104]:  # scroll up, keys 'w' or 'Up'
                self.__scroll_y('scroll', 
                                -1, 
                                'unit', 
                                event=event)
            elif event.keycode in [83, 40, 98]:  # scroll down, keys 's' or 'Down'
                self.__scroll_y('scroll',  
                                1, 
                                'unit', 
                                event=event
                                )
    def __returnreduction(self, size):
            if int(size) in range(0,7000):
                self.reduction = 4
            elif int(size) in range(7000,13000):
                self.reduction = 14
            elif int(size) in range(13000,19000): 
                self.reduction = 20
            elif int(size) in range(19000,26000):
                self.reduction = 45
            else:
                self.reduction = 60





from pynvn.path.ppath import refullpath,getdirpath
from pynvn.stringnvn import sepnumberandstrfromstr
from pynvn.stringnvn.slist import returnseplistintbbystr,returnlist_from_listinstr
from pynvn.excel import (cellcoordbyvalue,
                        lcellindexbyvalue,
                        col2num,
                        valuebyindexrowcell,
                        returnvaluekeyim,
                        repathlinkexcel,
                        relistsheet,
                        colnum_string,
                        rangecopyrefsamesheet
                        )
from pynvn.csv.tocsv import listtocsvbyarow
from pynvn.csv.tolist import convertcsvto1list
from tkinter import messagebox
class hexcel_sep:
    """copy excel to excel"""
    def __init__(self,wsheet = None,
                    dpath = None,
                    namefile = None,
                    dicrowconf = None,
                    wbnsct = None,
                    pathconf = None
                ):
                self.dicrowconf = dicrowconf
                self.dpath = dpath
                self.wsheet = wsheet
                self.namefile = namefile
                #max colum ws1
                self.mcol = self.wsheet.api.UsedRange.Columns.count
                #max row ws1
                self.mrow = self.wsheet.api.UsedRange.Rows.count
                self.dirpathconf = getdirpath(pathconf)
                self.__hm_rangege = (self.dicrowconf["hm_rangege"])
                self.numberhm = int(sepnumberandstrfromstr(self.__hm_rangege)[1])
                self.__wb1  = wbnsct
                listsheetnamehm = (self.dicrowconf["listsheetnamehm"])
                # return csv have list sheet name 
                self.pathlsn = refullpath(dirpath=self.dirpathconf,
                                        filename=listsheetnamehm)
                try:                        
                    self.lsheet = convertcsvto1list(path=self.pathlsn)
                except:
                    pass
    def habz30 (self):
        """handling data azb-30 sheet"""
        self.__hm_maxrow = int(self.dicrowconf["hm_maxrow"])
        self.__azb30_starcolumn = col2num(self.dicrowconf["azb30_starcolumn"])
        self.__azb30_rowhm = int(self.dicrowconf["azb30_rowhm"])
        self.__azb30_maxrowhm = int(self.dicrowconf["azb30_maxrowhm"])
        self.__azb30_startrowhm = int(self.dicrowconf["zab30_recor_l1"])
        self.__azb30_msa = (self.dicrowconf["azb30_ms"])
        lsheetnamehm = []
        lsheetnames = [sheet.name for sheet in self.__wb1.sheets ]
        for k in range (self.__azb30_starcolumn ,self.mcol):
            hmname =  self.wsheet.range(self.__azb30_rowhm , 
                                        k).value
            if hmname in lsheetnames:
                self.fomuluasfcol(k,hmname=hmname)
                lsheetnamehm.append(hmname)
        # convet to  csv file 
        listtocsvbyarow(listvalue=lsheetnamehm,
                        pathcsv=self.pathlsn )

    def fomuluasfcol (self,k,hmname = None ):
        """fomulas for column follow index"""
        pfile = repathlinkexcel(dpath=self.dpath,
                                namefile=self.namefile,
                                namesheet=hmname)
                                
        valueim = returnlist_from_listinstr(self.dicrowconf["zab30_valueim"].replace(":", ","))
        if len(valueim) != 0: 
            lindexrow_im = lcellindexbyvalue(max_row=self.mrow,
                                                min_row=self.__azb30_startrowhm,
                                                max_col=self.__azb30_msa,
                                                min_col=self.__azb30_msa,
                                                sheet=self.wsheet,
                                                lvalue=valueim
                                                )
        
            lvaluebyindecell_im = valuebyindexrowcell(lindexcell=lindexrow_im,
                                                    col=colnum_string(k),
                                                    sheet=self.wsheet)

        valueformulas = "=SUMIFS({0}!$L${4}:$L${3},{0}!$B${4}:$B${3},C{2})".format(pfile,
                                                                                    "'" + "AZB-30" + "'",
                                                                                    self.__azb30_startrowhm,
                                                                                    self.__hm_maxrow ,
                                                                                    self.numberhm
                                                                                    )

        rangecopyrefsamesheet(sheet=self.wsheet,
                            formulasfirstcell=valueformulas,
                            col_index=k,
                            startrow=self.__azb30_startrowhm,
                            endrow=self.__azb30_maxrowhm)
        if len (valueim) != 0: 
            returnvaluekeyim(cola=colnum_string(k),
                            listvalue_im=lvaluebyindecell_im,
                            sheet=self.wsheet,
                            indexrow_im=lindexrow_im
                            )
    def habz60 (self):
        """handling data azb-60 sheet"""
        col_msa = (self.dicrowconf["azb60_ms"])
        startrowhm = int(self.dicrowconf["zab60_recor_l1"])
        dongia = col2num(self.dicrowconf["azb60_dongia"])
        dongia_abc = self.dicrowconf["azb60_dongia"]
        rangeketcauthep = (self.dicrowconf["azb60_rangeketcauthep"])
        valueim = returnlist_from_listinstr(self.dicrowconf["zab60_valueim"].replace(":", ","))
        m_row = self.wsheet.range(col_msa + str(self.wsheet.cells.last_cell.row)).end('up').row
        # return index row by value
        if len (valueim) != 0: 
            lindexrow_im = lcellindexbyvalue(max_row=self.mrow,
                                                min_row=startrowhm,
                                                max_col=col_msa,
                                                min_col=col_msa,
                                                sheet=self.wsheet,
                                                lvalue=valueim
                                                )
        
            lvaluebyindecell_im = valuebyindexrowcell(lindexcell=lindexrow_im,
                                                    col=dongia_abc,
                                                    sheet=self.wsheet)

        # return range number
        rangese= returnseplistintbbystr(strint=rangeketcauthep)

        sumvalue = "=" + self.__returnsumvalue(iden="other",
                                        startrow60=startrowhm)

        rangecopyrefsamesheet(sheet=self.wsheet,
                            formulasfirstcell=sumvalue,
                            col_index=dongia,
                            startrow=startrowhm,
                            endrow=m_row)

        # rewwrite ket cau thep 
        sumvalue = "=" +  self.__returnsumvalue(iden="kct",
                                        startrow60=rangese[0])
        
        rangecopyrefsamesheet(sheet=self.wsheet,
                                formulasfirstcell=sumvalue,
                                col_index=dongia,
                                startrow=rangese[0],
                                endrow=rangese[1])
        if len (valueim) != 0: 
            returnvaluekeyim(cola=dongia_abc,
                            listvalue_im=lvaluebyindecell_im,
                            sheet=self.wsheet,
                            indexrow_im=lindexrow_im
                            )
  
    def __returnsumvalue (self,iden ="kct", startrow60 = 100):
        # list all sheet name from file path 
        sumvalue = ""
        for hmname in self.lsheet:

            valuesum = self.__valuecolsheet(i = startrow60,
                                            hmname=hmname,iden=iden)
            sumvalue = sumvalue + "+" +  valuesum        
        return sumvalue

    def __valuecolsheet (self,i = 1,hmname = None, iden = "kct"):
        """fomulas for column follow index"""
        pfile = repathlinkexcel(dpath=self.dpath,
                                namefile=self.namefile,
                                namesheet=hmname
                                )
        if iden != "kct":
            valueeee = 'SUMIF({0}!$BC:$BC,C{1},{0}!$CA:$CA) + SUMIF({0}!$BC:$BC,C{1},{0}!$CB:$CB)'.format(pfile,i)
        else:
            valueeee = 'SUMIF({0}!$BC:$BC,C{1},{0}!$CC:$CC) + SUMIF({0}!$BC:$BC,C{1},{0}!$CD:$CD)'.format(pfile,i)
        return valueeee

    def returnlistvaluebycolumnindex (self):

        return [cell.value for row in ws.iter_rows('C{}:C{}'.format(ws.min_row,ws.max_row)) for cell in row]
    def azb50(self):
        #='C:\Users\HP\Desktop\test\[KE HOACH NGAN SACH.xlsx]B3-Factory'!$BX$7
        startrowhm = int(self.dicrowconf["zab50_recor_l1"])
        maythicong_col =col2num(self.dicrowconf["azb50_hm"])
        dongia = self.dicrowconf["azb50_dongia"]
        dongia_int = col2num(dongia)
        colmay_nc = self.dicrowconf["azb50_colmay_nc"]
        colmay_nc_row = int(self.dicrowconf["azb50_colmay_nc_row"])

        crow = startrowhm + 1
        for sheetn in self.lsheet:
            self.wsheet.range(crow,maythicong_col).value = sheetn
            
            pfile = repathlinkexcel(dpath=self.dpath,
                                    namefile=self.namefile,
                                    namesheet=sheetn
                                    )
            self.wsheet.range(crow,dongia_int).value  = "={0}!${1}${2}".format(pfile,
                                                                            colmay_nc,
                                                                            colmay_nc_row)
            crow = crow + 1
    def azb40(self):
        startrowhm = int(self.dicrowconf["zab40_recor_l1"])
        maythicong_col =col2num(self.dicrowconf["azb40_hm"])
        dongia = self.dicrowconf["azb40_dongia"]
        dongia_int = col2num(dongia)
        colmay_nc = self.dicrowconf["azb40_colmay_nc"]
        colmay_nc_row = int(self.dicrowconf["azb40_colmay_nc_row"])
        crow = startrowhm + 1
        for sheetn in self.lsheet:
            self.wsheet.range(crow,maythicong_col).value = sheetn
            
            pfile = repathlinkexcel(dpath=self.dpath,
                                    namefile=self.namefile,
                                    namesheet=sheetn
                                    )
            self.wsheet.range(crow,dongia_int).value  = "={0}!${1}${2}".format(pfile,
                                                                            colmay_nc,
                                                                            colmay_nc_row)
            crow = crow + 1
    def itemiden(self):
        """item identification"""
        self.__azb30_starcolumn = col2num(self.dicrowconf["azb30_starcolumn"])
        self.__azb30_rowhm = int(self.dicrowconf["azb30_rowhm"])
        #lsheetnamehm = []
        lsheetnames = [sheet.name for sheet in self.__wb1.sheets ]
        lsheetnamehm = [self.wsheet.range(self.__azb30_rowhm , k).value  \
                        for k in range (self.__azb30_starcolumn ,self.mcol) \
                        if self.wsheet.range(self.__azb30_rowhm , k).value in lsheetnames]
        if len(lsheetnamehm) == 0:
            messagebox.showerror("Error", "Review {0}, has it processed?".format("AZB-30"))
        # convet to  csv file 
        listtocsvbyarow(listvalue=lsheetnamehm,
                        pathcsv=self.pathlsn )


from cryptography.fernet import Fernet

def write_key(path):
    """
    Generates a key and save it into a file
    """
    key = Fernet.generate_key()
    with open(path, "wb") as key_file:
        key_file.write(key)

def load_key(path):
    """
    Loads the key from the current directory named `key.key`
    """
    
    return open(path, "rb").read()

def encrypt(filename, 
            key, 
            nametow = None):
    """
    Given a filename (str) and key (bytes), it encrypts the file and write it
    """
    f = Fernet(key)
    encrypted_data = f.encrypt(nametow)
    # write the encrypted file
    with open(filename, "wb") as file:
        file.write(encrypted_data)

def decrypt(filename,key):
    """
    Given a filename (str) and key (bytes), it decrypts the file and write it
    """
    f = Fernet(key)
    with open(filename, "rb") as file:
        # read the encrypted data
        encrypted_data = file.read()
    # decrypt data
    decrypted_data = f.decrypt(encrypted_data)
    return decrypted_data.decode("utf-8")



import xlwings as xw
def cprange(pathtocopy = None,
            rangetocopy='BB5:CF100',
            pathtodes = None,
            rangetopaste="BB5", 
            namesheettocopy ='sheet_config' 
            ):

    """ 
    copy and paste range excel to another file
    ex: copy range [A1:C3] of pathtocopy to 
    C5:E20 of pathtodes
    copy to active sheet
    """
    pathtodes.range(rangetopaste).api.select
    wbtocopy = xw.Book(pathtocopy)
    sheet_copy = wbtocopy.sheets[namesheettocopy]
    sheet_copy.range(rangetocopy).api.copy
    pathtodes.api.paste
    # not cut excel 
    wbtocopy.app.api.CutCopyMode=False
    wbtocopy.close()
def cprangesamesheet(sheet = None,
                    rangetocopy='BB5:CF100',
                    rangetopaste="BB5"
                    ):

    """ 
    copy and paste range excel to same sheet in file
    """

    sheet_des.range("{0}{1}".format(abccol,
                                    startrow)).value = fomularex
    vtformulas = sheet_des.range("{0}{1}".format(abccol,
                                                startrow)).formula
    sheet_des.range("{0}{1}:{0}{2}".format(abccol,
                                            startrow,
                                            max_row)).formula = vtformulas



import itertools
""" 
class to handing data with symbol special 
import itertools
"""
class hstring:
    def  __init__(self, stringtohl = None,
                 igchar = None, 
                 path_cof = None,
                 chartorepign = None
                                        ):
        # ignore character     
        self.igchar = igchar 
        # path to conf
        self.path_cof = path_cof
        # string to handling 
        self.stringtohl = stringtohl
        #char to replace ignore character 
        self.chartorepign = chartorepign

    #string to remove char 
    def stringremoveigcharreturnspace(self):
        return self.stringtohl.translate({ord(i):self.chartorepign
                                             for i in self.igchar})

    #remove fist and end char 
    def revdupconstrfirstend(self):
        # check and replace fist string
        stfirst = self.replacecharinstrbyindex(0) if self.stringtohl[0]\
                                     in self.igchar  else self.stringtohl

        #set new parameter for stringtohl
        self.stringtohl = stfirst
        # check and replace end string
        stsecond = self.replacecharinstrbyindex(-1) if self.stringtohl[-1]\
                                 in self.igchar  else self.stringtohl
        return stsecond

    #replace char in by index from string, return string 
    def replacecharinstrbyindex (self, index):
        # convert string to list 
        tringh = list(self.stringtohl)
        #replace value index to chart replace 
        tringh[index] = self.chartorepign
        #convert to string
        st = "".join(tringh)
        return st

    # to remove continue return only single character , return string
    def removecontireturnsingle(self):
        tringh = list(self.stringtohl)
        i = 0 
        while i < len(tringh) - 1:
            if (tringh[i] in self.igchar and (tringh[i] == tringh[i+1])):
                del  tringh[i]
            else:
                i = i + 1
        st = "".join(tringh)
        return st

# return back to processed data, return string

"""
    stringtohl: string to handling 
    igchar : ignore character 
    chartorepign " character to replace 
"""
def returnvaluelist (stringtohl,
                        igchar,
                        chartorepign):
    

   hstring_hl =  hstring(stringtohl = stringtohl,
                                    igchar = igchar,
                                    chartorepign = chartorepign)

   hstring_hled = hstring_hl.stringremoveigcharreturnspace()
   # set new value for class
   hstring_hl.stringtohl = hstring_hled
   # set char to replace 
   hstring_hl.chartorepign = ""
   # remove char fist and end 
   hstring_hl1 = hstring_hl.revdupconstrfirstend()

   hstring_hl.stringtohl = hstring_hl1
   # set char to replace 
   hstring_hl.chartorepign = ","

   hstring_hl2 = hstring_hl.removecontireturnsingle()
   #python code to convert string to list 
   hstringlist = list(hstring_hl2.split(","))

   return hstringlist
#retl = returnvaluelist("(21:9),(21:11),(20:10),(22:10)",",.(:)",",")
#print ("retl",retl)

def grouper(n, iterable, fillvalue=None):
    "grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx"
    args = [iter(iterable)] * n
    return itertools.zip_longest(fillvalue=fillvalue, *args)
def duprowdata(df):
    for cols in df.columns:
        try:
            value = df.loc[1, cols]
            if value == None:
                value = ""       
        except:
            pass
        df[cols].fillna(value,inplace = True) 
    return df


class ratio:
    "caculate ratio"
    def __init__(self,real_w,real_h,w,h):
        self.w = w
        self.h = h
        self.real_w = real_w
        self.real_h = real_h
    
    def reratiomax(self):
        """return ratio max"""
        return max([self.real_w/self.w,self.real_h/self.h])

    def reratiomin(self):
        """return ratio min"""
        return min([self.real_w/self.w,self.real_h/self.h])


import csv
class wrcsv:
    def  __init__(self, pathtow = None,list = None,**kwargs):
        self.pathtow = pathtow
        self.args = list
        self.kwargs = kwargs

    def savevaltocsv(self):
        with open(self.pathtow, 'w') as csvFile:
            writer = csv.writer(csvFile,delimiter =',',lineterminator='\n')
            writer.writerows(self.args)
        csvFile.close()
    def ReDtallrowbyIndx (self,NumberRow):
            with open(self.pathtow,"r") as csvFile:
                readcsv =csv.reader(csvFile, delimiter=',')
                readcsv = list(readcsv)
                RowNumber = readcsv[NumberRow]
            csvFile.close()
            return RowNumber
    def writefilecsvFromRowArr(self):
        with open(self.pathtow , 'a') as csvFile:
            writer = csv.writer(csvFile,lineterminator='\n')
            writer.writerow(self.args)
        csvFile.close()
def pairlistinlisttocsv(listvalue = None, pathcsv = None):
    """ write row list to csv (pair list)
        ex : [[1, 4], [2, 7], [3, 8]] ---> csv file
    """
    with open(pathcsv, 'w') as csvFile:
        writer = csv.writer(csvFile,delimiter =',',lineterminator='\n')
        writer.writerows(listvalue)
    csvFile.close()
def listtocsvbyarow(listvalue = None, pathcsv = None):
    """ 
    write row list to csv (pair list)
    ex : [1,2,3,4,5,6] ---> csv file
    """
    with open(pathcsv, 'w') as csvFile:
        writer = csv.writer(csvFile,delimiter =',',lineterminator='\n')
        writer.writerow(listvalue)
    csvFile.close()



import csv
from pynvn.stringnvn import no_accent_vietnamese
def listocsvhor(pathtow = None,listv = None):
    """ convert list to csv horizontal """
    with open(pathtow, 'w') as csvFile:
        wr = csv.writer(csvFile,lineterminator='\n')
        wr.writerow(listv)
    csvFile.close()

def listocsvver(pathtow = None,listv = None):
    """ convert list to csv vertival """
    with open(pathtow, "w") as f:
        writer = csv.writer(f,lineterminator='\n')
        for row in listv:
            row = no_accent_vietnamese(row)
            writer.writerow([row])